@@ -98,7 +98,6 @@ static int print_phy_handler(struct nl_msg *msg, void *arg)
 		nla_parse(tb_band, NL80211_BAND_ATTR_MAX, nla_data(nl_band),
 			  nla_len(nl_band), NULL);
 
-#ifdef NL80211_BAND_ATTR_HT_CAPA
 		if (tb_band[NL80211_BAND_ATTR_HT_CAPA]) {
 			__u16 cap = nla_get_u16(tb_band[NL80211_BAND_ATTR_HT_CAPA]);
 			print_ht_capability(cap);
@@ -114,7 +113,10 @@ static int print_phy_handler(struct nl_msg *msg, void *arg)
 		if (tb_band[NL80211_BAND_ATTR_HT_MCS_SET] &&
 		    nla_len(tb_band[NL80211_BAND_ATTR_HT_MCS_SET]) == 16)
 			print_ht_mcs(nla_data(tb_band[NL80211_BAND_ATTR_HT_MCS_SET]));
-#endif
+		if (tb_band[NL80211_BAND_ATTR_VHT_CAPA] &&
+		    tb_band[NL80211_BAND_ATTR_VHT_MCS_SET])
+			print_vht_info(nla_get_u32(tb_band[NL80211_BAND_ATTR_VHT_CAPA]),
+				       nla_data(tb_band[NL80211_BAND_ATTR_VHT_MCS_SET]));
 
 		printf("\t\tFrequencies:\n");
 
@@ -432,12 +434,24 @@ broken_combination:
 	}
 
 	if (tb_msg[NL80211_ATTR_FEATURE_FLAGS]) {
-		if (nla_get_u32(tb_msg[NL80211_ATTR_FEATURE_FLAGS]) &
-				NL80211_FEATURE_SK_TX_STATUS)
+		unsigned int features = nla_get_u32(tb_msg[NL80211_ATTR_FEATURE_FLAGS]);
+
+		if (features & NL80211_FEATURE_SK_TX_STATUS)
 			printf("\tDevice supports TX status socket option.\n");
-		if (nla_get_u32(tb_msg[NL80211_ATTR_FEATURE_FLAGS]) &
-				NL80211_FEATURE_HT_IBSS)
+		if (features & NL80211_FEATURE_HT_IBSS)
 			printf("\tDevice supports HT-IBSS.\n");
+		if (features & NL80211_FEATURE_INACTIVITY_TIMER)
+			printf("\tDevice has client inactivity timer.\n");
+		if (features & NL80211_FEATURE_CELL_BASE_REG_HINTS)
+			printf("\tDevice accepts cell base station regulatory hints.\n");
+		if (features & NL80211_FEATURE_P2P_DEVICE_NEEDS_CHANNEL)
+			printf("\tP2P Device uses a channel (of the concurrent ones)\n");
+		if (features & NL80211_FEATURE_LOW_PRIORITY_SCAN)
+			printf("\tDevice supports low priority scan.\n");
+		if (features & NL80211_FEATURE_SCAN_FLUSH)
+			printf("\tDevice supports scan flush.\n");
+		if (features & NL80211_FEATURE_AP_SCAN)
+			printf("\tDevice supports AP scan.\n");
 	}
 
 	return NL_SKIP;
@@ -458,3 +472,16 @@ __COMMAND(NULL, info, "info", NULL, NL80211_CMD_GET_WIPHY, 0, 0, CIB_PHY, handle
 TOPLEVEL(list, NULL, NL80211_CMD_GET_WIPHY, NLM_F_DUMP, CIB_NONE, handle_info,
 	 "List all wireless devices and their capabilities.");
 TOPLEVEL(phy, NULL, NL80211_CMD_GET_WIPHY, NLM_F_DUMP, CIB_NONE, handle_info, NULL);
+
+static int handle_commands(struct nl80211_state *state,
+			   struct nl_cb *cb, struct nl_msg *msg,
+			   int argc, char **argv, enum id_input id)
+{
+	int i;
+	for (i = 1; i < NL80211_CMD_MAX; i++)
+		printf("%d (0x%x): %s\n", i, i, command_name(i));
+	/* don't send netlink messages */
+	return 2;
+}
+TOPLEVEL(commands, NULL, NL80211_CMD_GET_WIPHY, 0, CIB_NONE, handle_commands,
+	 "list all known commands and their decimal & hex value");
